# Start the imports
import wx

import LabelBook as LB
from Py2ExePanel import Py2ExePanel
from NotImplementedOrMissingPanel import NotImplementedOrMissingPanel

from Utilities import opj
from Constants import _lbStyle, _bookIcons


class AUINotebookPage(LB.FlatImageBook):

    def __init__(self, parent, project, compilers):
        """
        Default class constructor.

        @param parent: the parent widget
        @param project: the project associated to this book
        @param compilers: the available compilers on this machine

        """        

        # Initialize the custom FlatImageBook
        LB.FlatImageBook.__init__(self, parent, style=_lbStyle)

        self.MainFrame = wx.GetTopLevelParent(self)

        # Build the image list and add the pages to it
        self.BuildImageList()
        self.CreateBookPages(project, compilers)


    def BuildImageList(self):
        """ Builds the image list for FlatImageBook. """

        imgList = wx.ImageList(32, 32)
        # Maybe the icons are not that nice...
        for png in _bookIcons:
            imgList.Add(self.MainFrame.CreateBitmap(png))

        # Assign the image list to the book
        self.AssignImageList(imgList)


    def CreateBookPages(self, project, compilers):
        """ Creates the FlatImageBook pages. """

        # Loop over all the icons we have (4 at the moment)
        for ii, png in enumerate(_bookIcons):
            if ii == 0:
                # Is a py2exe, the only one implemented till now
                page = Py2ExePanel(self, project.GetName(), project.GetCreationDate())
            else:
                # No work dne on other compilers
                page = NotImplementedOrMissingPanel(self)

            # Add the page to FlatImageBook            
            self.AddPage(page, png, imageId=ii)

        # Select the first page, py2exe
        self.SetSelection(0)

        
    def SetTreeItem(self, treeItem):
        """ Stores the tree item associated with us. """

        self.treeItem = treeItem


    def GetTreeItem(self):
        """ Retrieves the tree item associated with us. """

        return self.treeItem


    def SetProject(self, project):
        """ Stores the project associated with us. """

        self.project = project

        keys = project.keys()
        # Loop over all the compilers and populate the pages
        for indx, key in enumerate(keys):
            self.GetPage(indx).SetConfiguration(project[key])


    def GetProject(self):
        """ Retrieves the project associated with us. """

        return self.project

