# Start the imports

import os
from Utilities import odict, opj, now


class Project(odict):

    def __init__(self, configuration=None, name=""):
        """
        Default class constructor.

        @param configuration: the project data
        @param name: the project name as entered by the user

        """        

        # Initialize the ordered dictionary
        # "Project" is a dictionary of dictionaries: the top level keys specify
        # which compiler the children keys refer to (i.e., py2exe, py2app etc...)
        # and for every top level key the secondary dictionary contains the
        # project configuration as entered by the user
        odict.__init__(self, configuration)

        # Store the data we are going to need later        
        self.name = name
        self.creationDate = now()
        self.customCode = ""
        self.hasBeenCompiled = False
        self.missingModules = []
        self.binaryDependencies = []


    def Update(self, compiler, keyName, keyValue):
        """ Updates the project with data inserted by the user. """

        self[compiler][keyName] = keyValue
                

    def SetConfiguration(self, compiler, configuration):
        """ Sets a whole configuration for the selected compiler. """

        self[compiler] = configuration


    def GetConfiguration(self, compiler):
        """ Returns the configuration for the selected compiler. """

        return self[compiler]


    def SetName(self, name):
        """ Sets the project name. """

        self.name = name
        

    def GetName(self):
        """ Returns the project name. """

        return self.name


    def GetCreationDate(self):
        """ Returns the project creation date. """

        return self.creationDate
    

    def SetCustomCode(self, code):
        """ Sets the custom code entered by the user. """

        self.customCode = code

        
    def GetCustomCode(self):
        """ Returns the custom code entered by the user. """

        return self.customCode


    def SetCompilationData(self, missingModules, binaryDependencies):
        """
        Sets the results of the compilation process, in terms of what the compiler
        says are the binary dependencies (dlls) and the missing modules (Python files).
        """

        self.missingModules = missingModules
        self.binaryDependencies = binaryDependencies


    def GetCompilationData(self):
        """ Returns the result of the compilation process. """

        return self.missingModules, self.binaryDependencies


    def HasBeenCompiled(self):
        """ Checks if a project has been compiled or not. """

        return self.hasBeenCompiled


    def GetExecutableName(self, compiler):
        """ Returns the executable name based on the chosen compiler. """

        configuration = self[compiler]
        # The executable name, for py2exe at least, lives in the folder:
        # /MainPythonFileFolder/dist_directory/
        fileName = configuration["script"]
        dist_dir = configuration["dist_dir"]
        buildDir, fileName = os.path.split(fileName)
        buildDir = opj(buildDir + "/" + dist_dir)
        exeName = os.path.splitext(fileName)[0] + ".exe"

        return opj(buildDir + "/" + exeName)


