# Start the imports

import os
import time

from UserDict import UserDict
from Constants import ListType

# This class keeps an ordered dictionary
class odict(UserDict):
    """ An ordered dictionary implementation. """    
    def __init__(self, dict=None):
        """ Default class constructor. """

        self._keys = []
        UserDict.__init__(self, dict)


    def __delitem__(self, key):

        UserDict.__delitem__(self, key)
        self._keys.remove(key)

    def __setitem__(self, key, item):

        UserDict.__setitem__(self, key, item)
        if key not in self._keys: self._keys.append(key)


    def clear(self):

        UserDict.clear(self)
        self._keys = []


    def copy(self):

        dict = UserDict.copy(self)
        dict._keys = self._keys[:]
        return dict


    def items(self):

        return zip(self._keys, self.values())


    def keys(self):

        return self._keys


    def popitem(self):

        try:
            key = self._keys[-1]
        except IndexError:
            raise KeyError('dictionary is empty')

        val = self[key]
        del self[key]

        return (key, val)


    def setdefault(self, key, failobj=None):

        UserDict.setdefault(self, key, failobj)
        if key not in self._keys: self._keys.append(key)


    def update(self, dict):

        UserDict.update(self, dict)
        for key in dict.keys():
            if key not in self._keys: self._keys.append(key)


    def values(self):

        return map(self.get, self._keys)


# Path and file filling (os indipendent)
def opj(path):
    """Convert paths to the platform-specific separator"""

    str = apply(os.path.join, tuple(path.split('/')))
    # HACK: on Linux, a leading / gets lost...
    if path.startswith('/'):
        str = '/' + str
    return str


def flatten(list):
    """ Internal function that flattens a ND list. """

    res = []
    for item in list:
        if type(item) == ListType:
            res = res + flatten(item)
        elif item is not None:
            res = res + [item]
    return res


def unique(list):
    """ Internal function, returning the unique elements in a list."""

    # Create a fake dictionary
    res = {}

    for item in list:
        # Loop over all the items in the list
        key, value = item
        if res.has_key(key):
            res[key].append(value)
        else:
            res[key] = [value]

    # Return the dictionary values (a list)    
    return res.items()


def setupString(key, item):
    """ Sets up the strings for py2exe. """

    # This is an incredible and horrible hack, and it may not always work     
    if key == "data_files":

        text = "["
        for i in xrange(len(item)):
            indent = len(key) + 9 + len(item[i][0])
            text += "('%s', "%item[i][0] + (",\n" + " "*indent).join(repr(item[i][1]).split(",")) + ")"
            if i != len(item)-1:
                text += ",\n" + " "*(len(key)+4)
        item = text + "]"

    else:
        
        indent = len(key)+3
        spacer = "\n" + " "*indent
        item = ("%s"%item).split(",")

        text = ""
        lentext = 0
        for chunk in item:
            text += chunk + ","
            lentext += len(chunk)+2
            if lentext > 60 and chunk != item[-1]:
                text += spacer
                lentext = len(spacer)-1

        item = text[0:-1]
        
    return item


def now():
    """ Returns the current time formatted. """

    t = time.localtime(time.time())
    st = time.strftime("%d %B %Y @ %H:%M:%S", t)
    
    return st


def shortNow():
    """ Returns the current time formatted. """
    
    t = time.localtime(time.time())
    st = time.strftime("%H:%M:%S", t)

    return st

